@@ -1,3 +1,4 @@
+from ._integrand import Integrand
 from .asian_option import AsianOption
 from .european_option import EuropeanOption
 from .keister import Keister
@@ -5,4 +6,5 @@ from .linear0 import Linear0
 from .custom_fun import CustomFun
 from .ml_call_options import MLCallOptions
 from .box_integral import BoxIntegral
-from .sobol_indices import SobolIndices
\ No newline at end of file
+from .sobol_indices import SobolIndices,SensitivityIndices
+from .ishigami import Ishigami
