@@ -1,10 +1,10 @@
 from ._stopping_criterion import StoppingCriterion
 from ..accumulate_data import MeanVarData
 from ..discrete_distribution._discrete_distribution import DiscreteDistribution
-from ..integrand import Keister
-from ..true_measure import Gaussian
+from ..integrand import Keister, CustomFun
+from ..true_measure import Gaussian, Uniform
 from ..discrete_distribution import IIDStdUniform
-from ..util import _tol_fun, MaxSamplesWarning, NotYetImplemented
+from ..util import _tol_fun, MaxSamplesWarning
 from numpy import *
 from scipy.optimize import fsolve
 from scipy.stats import norm
@@ -13,40 +13,46 @@ import warnings
 
 
 class CubMCG(StoppingCriterion):
-    """
+    r"""
     Stopping criterion with guaranteed accuracy.
 
-    >>> k = Keister(Gaussian(IIDStdUniform(2,seed=7),covariance=1./2))
+    >>> k = Keister(IIDStdUniform(2,seed=7))
     >>> sc = CubMCG(k,abs_tol=.05)
     >>> solution,data = sc.integrate()
-    >>> solution
-    1.803926962264685
     >>> data
-    Solution: 1.8039         
-    Keister (Integrand Object)
-    IIDStdUniform (DiscreteDistribution Object)
-        dimension       2^(1)
-        seed            7
-        mimics          StdUniform
-    Gaussian (TrueMeasure Object)
-        mean            0
-        covariance      2^(-1)
-        decomp_type     pca
+    MeanVarData (AccumulateData Object)
+        solution        1.807
+        error_bound     0.050
+        n_total         15256
+        n               14232
+        levels          1
+        time_integrate  ...
     CubMCG (StoppingCriterion Object)
-        inflate         1.200
-        alpha           0.010
         abs_tol         0.050
         rel_tol         0
         n_init          2^(10)
         n_max           10000000000
-    MeanVarData (AccumulateData Object)
-        levels          1
-        solution        1.804
-        n               13473
-        n_total         14497
-        error_bound     0.050
-        confid_int      [1.754 1.854]
-        time_integrate  ...
+        inflate         1.200
+        alpha           0.010
+    Keister (Integrand Object)
+    Gaussian (TrueMeasure Object)
+        mean            0
+        covariance      2^(-1)
+        decomp_type     PCA
+    IIDStdUniform (DiscreteDistribution Object)
+        d               2^(1)
+        entropy         7
+        spawn_key       ()
+    >>> dd = IIDStdUniform(1,seed=7)
+    >>> k = Keister(dd)
+    >>> cv1 = CustomFun(Uniform(dd),lambda x: sin(pi*x).sum(1))
+    >>> cv1mean = 2/pi
+    >>> cv2 = CustomFun(Uniform(dd),lambda x: (-3*(x-.5)**2+1).sum(1))
+    >>> cv2mean = 3/4
+    >>> sc1 = CubMCG(k,abs_tol=.05,control_variates=[cv1,cv2],control_variate_means=[cv1mean,cv2mean])
+    >>> sol,data = sc1.integrate()
+    >>> sol
+    1.384...
 
     Original Implementation:
 
@@ -73,9 +79,8 @@ class CubMCG(StoppingCriterion):
         inequality would be satisfied: $\P(| mu - tmu | \\le tol\_fun) \\ge 1 - \\alpha$.
     """
 
-    parameters = ['inflate','alpha','abs_tol','rel_tol','n_init','n_max']
     def __init__(self, integrand, abs_tol=1e-2, rel_tol=0., n_init=1024., n_max=1e10,
-                 inflate=1.2, alpha=0.01):
+                 inflate=1.2, alpha=0.01, control_variates=[], control_variate_means=[]):
         """
         Args:
             integrand (Integrand): an instance of Integrand
@@ -85,7 +90,12 @@ class CubMCG(StoppingCriterion):
             rel_tol: relative error tolerance
             n_init: initial number of samples
             n_max: maximum number of samples
+            control_variates (list): list of integrand objects to be used as control variates. 
+                Control variates are currently only compatible with single level problems. 
+                The same discrete distribution instance must be used for the integrand and each of the control variates. 
+            control_variate_means (list): list of means for each control variate
         """
+        self.parameters = ['abs_tol','rel_tol','n_init','n_max','inflate','alpha']
         # Set Attributes
         self.abs_tol = float(abs_tol)
         self.rel_tol = float(rel_tol)
@@ -97,17 +107,23 @@ class CubMCG(StoppingCriterion):
         self.kurtmax = (n_init - 3) / (n_init - 1) + \
             (self.alpha_sigma * n_init) / (1 - self.alpha_sigma) * \
             (1 - 1. / self.inflate**2)**2
+        # QMCPy Objs
         self.integrand = integrand
+        self.true_measure = self.integrand.true_measure
+        self.discrete_distrib = self.integrand.discrete_distrib
+        self.cv = control_variates
+        self.cv_mu = control_variate_means
         # Verify Compliant Construction
-        distribution = integrand.measure.distribution
         allowed_levels = ['single']
-        allowed_distribs = ["IIDStdUniform", "IIDStdGaussian", "CustomIIDDistribution"]
-        super(CubMCG,self).__init__(distribution, integrand, allowed_levels, allowed_distribs)
+        allowed_distribs = [IIDStdUniform]
+        allow_vectorized_integrals = False
+        super(CubMCG,self).__init__(allowed_levels, allowed_distribs, allow_vectorized_integrals)
 
     def integrate(self):
         """ See abstract method. """
         # Construct AccumulateData Object to House Integration data
-        self.data = MeanVarData(self, self.integrand, self.n_init)  # house integration data
+        self.data = MeanVarData(self, self.integrand, self.true_measure, self.discrete_distrib, 
+            self.n_init, self.cv, self.cv_mu)  # house integration data
         t_start = time()
         # Pilot Sample
         self.data.update_data()
