@@ -1,10 +1,9 @@
-from copy import deepcopy
 from ._stopping_criterion import StoppingCriterion
 from ..accumulate_data import MeanVarDataRep
 from ..discrete_distribution._discrete_distribution import DiscreteDistribution
 from ..discrete_distribution import Lattice,DigitalNetB2,Halton
-from ..true_measure import Gaussian
-from ..integrand import Keister,BoxIntegral
+from ..true_measure import Gaussian,Uniform
+from ..integrand import Keister,BoxIntegral,CustomFun
 from ..util import MaxSamplesWarning, NotYetImplemented, ParameterWarning, ParameterError
 from numpy import *
 from scipy.stats import norm
@@ -24,17 +23,16 @@ class CubQMCCLT(StoppingCriterion):
     >>> data
     MeanVarDataRep (AccumulateData Object)
         solution        1.380
-        indv_error_bound 6.92e-04
+        indv_error      6.92e-04
         ci_low          1.380
         ci_high         1.381
         ci_comb_low     1.380
         ci_comb_high    1.381
-        solution_comb   1.380
         flags_comb      0
         flags_indv      0
         n_total         2^(12)
-        n               2^(8)
-        replications    2^(4)
+        n               2^(12)
+        n_rep           2^(8)
         time_integrate  ...
     CubQMCCLT (StoppingCriterion Object)
         inflate         1.200
@@ -43,6 +41,7 @@ class CubQMCCLT(StoppingCriterion):
         rel_tol         0
         n_init          2^(8)
         n_max           2^(30)
+        replications    2^(4)
     Keister (Integrand Object)
     Gaussian (TrueMeasure Object)
         mean            0
@@ -64,17 +63,16 @@ class CubQMCCLT(StoppingCriterion):
     >>> data
     MeanVarDataRep (AccumulateData Object)
         solution        [1.19  0.961]
-        indv_error_bound [0.001 0.001]
+        indv_error      [0.001 0.001]
         ci_low          [1.19 0.96]
         ci_high         [1.191 0.961]
         ci_comb_low     [1.19 0.96]
         ci_comb_high    [1.191 0.961]
-        solution_comb   [1.19  0.961]
         flags_comb      [False False]
         flags_indv      [False False]
         n_total         2^(21)
-        n               [131072.    512.]
-        replications    2^(4)
+        n               [2097152.    8192.]
+        n_rep           [131072.    512.]
         time_integrate  ...
     CubQMCCLT (StoppingCriterion Object)
         inflate         1.200
@@ -83,6 +81,7 @@ class CubQMCCLT(StoppingCriterion):
         rel_tol         0
         n_init          2^(8)
         n_max           2^(30)
+        replications    2^(4)
     BoxIntegral (Integrand Object)
         s               [-1  1]
     Uniform (TrueMeasure Object)
@@ -100,6 +99,54 @@ class CubQMCCLT(StoppingCriterion):
     >>> true_value = array([sol3neg1,sol31])
     >>> (abs(true_value-solution)<abs_tol).all()
     True
+    >>> cf = CustomFun(
+    ...     true_measure = Uniform(DigitalNetB2(6,seed=7)),
+    ...     g = lambda x,compute_flags=None: (2*arange(1,7)*x).reshape(-1,2,3),
+    ...     dprime = (2,3))
+    >>> sol,data = CubQMCCLT(cf,abs_tol=1e-4).integrate()
+    >>> data
+    MeanVarDataRep (AccumulateData Object)
+        solution        [[1. 2. 3.]
+                        [4. 5. 6.]]
+        indv_error      [[2.484e-05 0.000e+00 0.000e+00]
+                        [5.708e-06 2.178e-10 0.000e+00]]
+        ci_low          [[1. 2. 3.]
+                        [4. 5. 6.]]
+        ci_high         [[1. 2. 3.]
+                        [4. 5. 6.]]
+        ci_comb_low     [[1. 2. 3.]
+                        [4. 5. 6.]]
+        ci_comb_high    [[1. 2. 3.]
+                        [4. 5. 6.]]
+        flags_comb      [[False False False]
+                        [False False False]]
+        flags_indv      [[False False False]
+                        [False False False]]
+        n_total         2^(14)
+        n               [[ 4096.  4096.  4096.]
+                        [16384.  4096.  4096.]]
+        n_rep           [[ 256.  256.  256.]
+                        [1024.  256.  256.]]
+        time_integrate  ...
+    CubQMCCLT (StoppingCriterion Object)
+        inflate         1.200
+        alpha           0.010
+        abs_tol         1.00e-04
+        rel_tol         0
+        n_init          2^(8)
+        n_max           2^(30)
+        replications    2^(4)
+    CustomFun (Integrand Object)
+    Uniform (TrueMeasure Object)
+        lower_bound     0
+        upper_bound     1
+    DigitalNetB2 (DiscreteDistribution Object)
+        d               6
+        dvec            [0 1 2 3 4 5]
+        randomize       LMS_DS
+        graycode        0
+        entropy         7
+        spawn_key       ()
     """
 
     def __init__(self, integrand, abs_tol=1e-2, rel_tol=0., n_init=256., n_max=2**30,
@@ -118,7 +165,7 @@ class CubQMCCLT(StoppingCriterion):
                 absolute tolerance, and relative tolerance which returns the approximate error. 
                 Default indicates integration until either absolute OR relative tolerance is satisfied.
         """
-        self.parameters = ['inflate','alpha','abs_tol','rel_tol','n_init','n_max']
+        self.parameters = ['inflate','alpha','abs_tol','rel_tol','n_init','n_max','replications']
         # Input Checks
         if log2(n_init) % 1 != 0:
             warning_s = ' n_init must be a power of 2. Using n_init = 32'
@@ -132,12 +179,14 @@ class CubQMCCLT(StoppingCriterion):
         self.alpha = float(alpha)
         self.z_star = -norm.ppf(self.alpha / 2)
         self.inflate = float(inflate)
-        self.replications = replications
+        self.replications = int(replications)
         self.error_fun = error_fun
         # QMCPy Objs
         self.integrand = integrand
         self.true_measure = self.integrand.true_measure
         self.discrete_distrib = self.integrand.discrete_distrib
+        self.dprime = self.integrand.dprime
+        self.d = self.discrete_distrib.d
         # Verify Compliant Construction
         allowed_levels = ["single"]
         allowed_distribs = [Lattice,DigitalNetB2,Halton]
@@ -148,43 +197,79 @@ class CubQMCCLT(StoppingCriterion):
          
     def integrate(self):
         """ See abstract method. """
-        # Construct AccumulateData Object to House Integration data
-        self.data = MeanVarDataRep(self, self.integrand, self.true_measure, self.discrete_distrib, self.n_init, self.replications)
         t_start = time()
+        self.datum = empty(self.dprime,dtype=object)
+        for j in ndindex(self.dprime):
+            self.datum[j] = MeanVarDataRep(self.z_star,self.inflate,self.replications)
+        self.data = MeanVarDataRep.__new__(MeanVarDataRep)
+        self.data.flags_indv = tile(True,self.dprime)
+        self.data.rep_distribs = self.integrand.discrete_distrib.spawn(s=self.replications)
+        self.data.n_rep = tile(self.n_init,self.dprime)
+        self.data.n_min_rep = 0
+        self.data.ci_low = tile(-inf,self.dprime)
+        self.data.ci_high = tile(inf,self.dprime)
+        self.data.solution_indv = tile(nan,self.dprime)
+        self.data.solution = nan
+        self.data.xfull = empty((0,self.d))
+        self.data.yfull = empty((0,)+self.dprime)
         while True:
-            self.data.update_data()
-            self.data.indv_error_bound = self.z_star * self.inflate * self.data.sighat / sqrt(self.data.replications)
-            self.data.ci_low = self.data.solution_indv-self.data.indv_error_bound
-            self.data.ci_high = self.data.solution_indv+self.data.indv_error_bound
+            n_min = self.data.n_min_rep
+            n_max = int(self.data.n_rep.max())
+            n = int(n_max-n_min)
+            xnext = vstack([self.data.rep_distribs[r].gen_samples(n_min=n_min,n_max=n_max) for r in range(self.replications)])
+            ynext = self.integrand.f(xnext,compute_flags=self.data.flags_indv)
+            for j in ndindex(self.dprime):
+                if not self.data.flags_indv[j]: continue
+                yj = ynext[(slice(None),)+j].reshape((n,self.replications),order='f')
+                self.data.solution_indv[j],self.data.ci_low[j],self.data.ci_high[j] = self.datum[j].update_data(yj)
+            self.data.xfull = vstack((self.data.xfull,xnext))
+            self.data.yfull = vstack((self.data.yfull,ynext))
+            self.data.indv_error = (self.data.ci_high-self.data.ci_low)/2
             self.data.ci_comb_low,self.data.ci_comb_high,self.data.violated = self.integrand.bound_fun(self.data.ci_low,self.data.ci_high)
             error_low = self.error_fun(self.data.ci_comb_low,self.abs_tol,self.rel_tol)
             error_high = self.error_fun(self.data.ci_comb_high,self.abs_tol,self.rel_tol)
-            self.data.solution_comb = 1/2*(self.data.ci_comb_low+self.data.ci_comb_high+error_low-error_high)
-            rem_error_low = abs(self.data.ci_comb_low-self.data.solution_comb)-error_low
-            rem_error_high = abs(self.data.ci_comb_high-self.data.solution_comb)-error_high
+            self.data.solution = 1/2*(self.data.ci_comb_low+self.data.ci_comb_high+error_low-error_high)
+            rem_error_low = abs(self.data.ci_comb_low-self.data.solution)-error_low
+            rem_error_high = abs(self.data.ci_comb_high-self.data.solution)-error_high
             self.data.flags_comb = maximum(rem_error_low,rem_error_high)>=0
             self.data.flags_comb |= self.data.violated
             self.data.flags_indv = self.integrand.dependency(self.data.flags_comb)
+            self.data.n = self.replications*self.data.n_rep
+            self.data.n_total = self.data.n.max()
             if sum(self.data.flags_indv)==0:
-                # sufficiently estimated
-                break
-            elif 2 * self.data.n_total > self.n_max:
-                # doubling samples would go over n_max
+                break # sufficiently estimated
+            elif 2*self.data.n_total>self.n_max:
                 warning_s = """
                 Alread generated %d samples.
                 Trying to generate %d new samples would exceeds n_max = %d.
                 No more samples will be generated.
-                Note that error tolerances may not be satisfied""" \
-                % (int(self.data.n_total), int(self.data.n_total), int(self.n_max))
+                Note that error tolerances may not be satisfied. """ \
+                % (int(self.data.n_total),int(self.data.n_total),int(self.n_max))
                 warnings.warn(warning_s, MaxSamplesWarning)
                 break
             else:
-                # double sample size
-                self.data.nprev = where(self.data.flags_indv,self.data.n,self.data.nprev)
-                self.data.n = where(self.data.flags_indv,2*self.data.n,self.data.n)
-        self.data.time_integrate = time() - t_start
-        self.data.solution = self.data.solution_comb
-        return self.data.solution, self.data
+                self.data.n_min_rep = n_max
+                self.data.n_rep += self.data.n_rep*self.data.flags_indv # double sample size
+        self.data.integrand = self.integrand
+        self.data.true_measure = self.true_measure
+        self.data.discrete_distrib = self.discrete_distrib
+        self.data.stopping_crit = self
+        self.data.parameters = [
+            'solution',
+            'indv_error',
+            'ci_low',
+            'ci_high',
+            'ci_comb_low',
+            'ci_comb_high',
+            'flags_comb',
+            'flags_indv',
+            'n_total',
+            'n',
+            'n_rep',
+            'time_integrate']
+        self.data.datum = self.datum
+        self.data.time_integrate = time()-t_start
+        return self.data.solution,self.data
     
     def set_tolerance(self, abs_tol=None, rel_tol=None):
         """
